@@ -25,6 +25,7 @@ import tempfile
 
 from ..utils import requireUnicode, chunkCopy, datePicker
 from .. import core
+from ..core import TXXX_ALBUM_TYPE, TXXX_ARTIST_ORIGIN, ALBUM_TYPE_IDS
 from .. import Error
 from . import (ID3_ANY_VERSION, ID3_V1, ID3_V1_0, ID3_V1_1,
                ID3_V2, ID3_V2_2, ID3_V2_3, ID3_V2_4, versionToString)
@@ -245,6 +246,13 @@ class Tag(core.Tag):
     def _getArtist(self):
         return self.getTextFrame(frames.ARTIST_FID)
 
+    @requireUnicode(1)
+    def _setAlbumArtist(self, val):
+        self.setTextFrame(frames.ALBUM_ARTIST_FID, val)
+
+    def _getAlbumArtist(self):
+        return self.getTextFrame(frames.ALBUM_ARTIST_FID)
+
     @requireUnicode(1)
     def _setAlbum(self, val):
         self.setTextFrame(frames.ALBUM_FID, val)
@@ -725,16 +733,22 @@ class Tag(core.Tag):
         else:
             self.frame_set[frames.TOS_FID] = frames.TermsOfUseFrame(text=tos)
 
+    def _raiseIfReadonly(self):
+        if self.read_only:
+            raise RuntimeError("Tag is set read only.")
+
     def save(self, filename=None, version=None, encoding=None, backup=False,
-             preserve_file_time=False):
+             preserve_file_time=False, max_padding=None):
         '''Save the tag. If ``filename`` is not give the value from the
         ``file_info`` member is used, or a ``TagException`` is raised. The
         ``version`` argument can be used to select an ID3 version other than
         the version read. ``Select text encoding with ``encoding`` or use
         the existing (or default) encoding. If ``backup`` is True the orignal
         file is preserved; likewise if ``preserve_file_time`` is True the
-        file's modification/access times are not updated.
+        file´s modification/access times are not updated.
         '''
+        self._raiseIfReadonly()
+
         if not (filename or self.file_info):
             raise TagException("No file")
         elif filename:
@@ -756,18 +770,19 @@ class Tag(core.Tag):
         if version[0] == 1:
             self._saveV1Tag(version)
         elif version[0] == 2:
-            self._saveV2Tag(version, encoding)
+            self._saveV2Tag(version, encoding, max_padding)
         else:
             assert(not "Version bug: %s" % str(version))
 
         if preserve_file_time and None not in (self.file_info.atime,
                                                self.file_info.mtime):
-            os.utime(self.file_info.name,
-                     (self.file_info.atime, self.file_info.mtime))
+            self.file_info.touch((self.file_info.atime, self.file_info.mtime))
         else:
-            self.file_info.mtime = os.stat(self.file_info.name).st_mtime
+            self.file_info.initStatTimes()
 
     def _saveV1Tag(self, version):
+        self._raiseIfReadonly()
+
         assert(version[0] == 1)
 
         def pack(s, n):
@@ -802,7 +817,7 @@ class Tag(core.Tag):
         tag += cmt
 
         if not self.genre or self.genre.id is None:
-            genre = 0
+            genre = 12  # Other
         else:
             genre = self.genre.id
         tag += chr(genre & 0xff)
@@ -825,7 +840,7 @@ class Tag(core.Tag):
             tag_file.write(tag)
             tag_file.flush()
 
-    def _render(self, version, curr_tag_size):
+    def _render(self, version, curr_tag_size, max_padding_size):
         std_frames = []
         non_std_frames = []
         for f in self.frame_set.getAllFrames():
@@ -883,11 +898,15 @@ class Tag(core.Tag):
         padding_size = 0
         if pending_size > curr_tag_size:
             # current tag (minus padding) larger than the current (plus padding)
-            rewrite_required = True
             padding_size = DEFAULT_PADDING
+            rewrite_required = True
         else:
-            rewrite_required = False
             padding_size = curr_tag_size - pending_size
+            if max_padding_size is not None and padding_size > max_padding_size:
+                padding_size = min(DEFAULT_PADDING, max_padding_size)
+                rewrite_required = True
+            else:
+                rewrite_required = False
 
         assert(padding_size >= 0)
         log.debug("Using %d bytes of padding" % padding_size)
@@ -916,8 +935,11 @@ class Tag(core.Tag):
         assert(len(tag_data) == (total_size - padding_size))
         return (rewrite_required, tag_data, "\x00" * padding_size)
 
-    def _saveV2Tag(self, version, encoding):
+    def _saveV2Tag(self, version, encoding, max_padding):
+        self._raiseIfReadonly()
+
         assert(version[0] == 2 and version[1] != 2)
+
         log.debug("Rendering tag version: %s" % versionToString(version))
 
         file_exists = os.path.exists(self.file_info.name)
@@ -941,13 +963,14 @@ class Tag(core.Tag):
                 log.debug("Current tag size: %d" % curr_tag_size)
 
             rewrite_required, tag_data, padding = self._render(version,
-                                                               curr_tag_size)
+                                                               curr_tag_size,
+                                                               max_padding)
             log.debug("Writing %d bytes of tag data and %d bytes of "
                       "padding" % (len(tag_data), len(padding)))
             if rewrite_required:
                 # Open tmp file
-                tmp_name = tempfile.mktemp()
-                with open(tmp_name, "wb") as tmp_file:
+                with tempfile.NamedTemporaryFile("wb", delete=False) \
+                        as tmp_file:
                     tmp_file.write(tag_data + padding)
 
                     # Copy audio data in chunks
@@ -961,16 +984,18 @@ class Tag(core.Tag):
                         tag_file.seek(seek_point)
                         chunkCopy(tag_file, tmp_file)
 
+                    tmp_file.flush()
+
                 # Move tmp to orig.
-                shutil.copyfile(tmp_name, self.file_info.name)
-                os.unlink(tmp_name)
+                shutil.copyfile(tmp_file.name, self.file_info.name)
+                os.unlink(tmp_file.name)
 
             else:
                 with open(self.file_info.name, "r+b") as tag_file:
                     tag_file.write(tag_data + padding)
 
         else:
-            _, tag_data, padding = self._render(version, 0)
+            _, tag_data, padding = self._render(version, 0, None)
             with open(self.file_info.name, "wb") as tag_file:
                 tag_file.write(tag_data + padding)
 
@@ -1118,13 +1143,13 @@ class Tag(core.Tag):
                     tag_file.seek(tag.file_info.tag_size)
 
                     # Open tmp file
-                    tmp_name = tempfile.mktemp()
-                    with open(tmp_name, "wb") as tmp_file:
+                    with tempfile.NamedTemporaryFile("wb", delete=False) \
+                            as tmp_file:
                         chunkCopy(tag_file, tmp_file)
 
                     # Move tmp to orig
-                    shutil.copyfile(tmp_name, filename)
-                    os.unlink(tmp_name)
+                    shutil.copyfile(tmp_file.name, filename)
+                    os.unlink(tmp_file.name)
 
                     retval |= True
 
@@ -1138,6 +1163,46 @@ class Tag(core.Tag):
     def table_of_contents(self):
         return self._tocs
 
+    @property
+    def album_type(self):
+        if TXXX_ALBUM_TYPE in self.user_text_frames:
+            return self.user_text_frames.get(TXXX_ALBUM_TYPE).text
+        else:
+            return None
+
+    @album_type.setter
+    def album_type(self, t):
+        if not t:
+            self.user_text_frames.remove(TXXX_ALBUM_TYPE)
+        elif t in ALBUM_TYPE_IDS:
+            self.user_text_frames.set(t, TXXX_ALBUM_TYPE)
+        else:
+            raise ValueError("Invalid album_type: %s" % t)
+
+    @property
+    def artist_origin(self):
+        '''Returns a 3-tuple: (city, state, country) Any may be ``None``.'''
+        if TXXX_ARTIST_ORIGIN in self.user_text_frames:
+            origin = self.user_text_frames.get(TXXX_ARTIST_ORIGIN).text
+            vals = origin.split('\t')
+        else:
+            vals = [None] * 3
+
+        vals.extend([None] * (3 - len(vals)))
+        vals = [None if not v else v for v in vals]
+        assert(len(vals) == 3)
+        return vals
+
+    @artist_origin.setter
+    def artist_origin(self, city, state, country):
+        vals = (city, state, country)
+        vals = [None if not v else v for v in vals]
+        if vals == (None, None, None):
+            self.user_text_frames.remove(TXXX_ARTIST_ORIGIN)
+        else:
+            assert(len(vals) == 3)
+            self.user_text_frames.set('\t'.join(vals), TXXX_ARTIST_ORIGIN)
+
 
 class FileInfo:
     '''
@@ -1163,6 +1228,9 @@ class FileInfo:
         self.tag_size = 0  # This includes the padding byte count.
         self.tag_padding_size = 0
 
+        self.initStatTimes()
+
+    def initStatTimes(self):
         try:
             s = os.stat(self.name)
         except OSError:
@@ -1170,6 +1238,11 @@ class FileInfo:
         else:
             self.atime, self.mtime = s.st_atime, s.st_mtime
 
+    def touch(self, times):
+        '''times is a 2-tuple of (atime, mtime).'''
+        os.utime(self.name, times)
+        self.initStatTimes()
+
 
 class AccessorBase(object):
     def __init__(self, fid, fs, match_func=None):
@@ -1389,6 +1462,10 @@ class UserTextsAccessor(AccessorBase):
     def get(self, description):
         return super(UserTextsAccessor, self).get(description)
 
+    @requireUnicode(1)
+    def __contains__(self, description):
+        return bool(self.get(description))
+
 
 class UniqueFileIdAccessor(AccessorBase):
     def __init__(self, fs):
@@ -1666,6 +1743,7 @@ class TagTemplate(string.Template):
 
     def _makeMapping(self, tag, zeropad):
         return {"artist": tag.artist if tag else None,
+                "album_artist": tag.album_artist if tag else None,
                 "album": tag.album if tag else None,
                 "title": tag.title if tag else None,
                 "track:num": (self._track, zeropad) if tag else None,
