@@ -33,6 +33,7 @@ log = logging.getLogger(__name__)
 
 FIELD_DELIM = ':'
 
+DEFAULT_MAX_PADDING = 64*1024
 
 class ClassicPlugin(LoaderPlugin):
     SUMMARY = u"Classic eyeD3 interface for viewing and editing tags."
@@ -65,6 +66,9 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
                        metavar="STRING", help=ARGS_HELP["--artist"])
         g.add_argument("-A", "--album", type=UnicodeArg, dest="album",
                        metavar="STRING", help=ARGS_HELP["--album"])
+        g.add_argument("-b", "--album-artist", type=UnicodeArg,
+                       dest="album_artist", metavar="STRING",
+                       help=ARGS_HELP["--album-artist"])
         g.add_argument("-t", "--title", type=UnicodeArg, dest="title",
                        metavar="STRING", help=ARGS_HELP["--title"])
         g.add_argument("-n", "--track", type=PositiveIntArg, dest="track",
@@ -72,6 +76,10 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
         g.add_argument("-N", "--track-total", type=PositiveIntArg,
                        dest="track_total", metavar="NUM",
                        help=ARGS_HELP["--track-total"])
+
+        g.add_argument("--track-offset", type=int, dest="track_offset",
+                       metavar="N", help=ARGS_HELP["--track-offset"])
+
         g.add_argument("-d", "--disc-num", type=PositiveIntArg, dest="disc_num",
                        metavar="NUM", help=ARGS_HELP["--disc-num"])
         g.add_argument("-D", "--disc-total", type=PositiveIntArg,
@@ -384,6 +392,14 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
                           dest="remove_fids", metavar="FID",
                           help=ARGS_HELP["--remove-frame"])
 
+        # 'True' means 'apply default max_padding, but only if saving anyhow'
+        gid3.add_argument("--max-padding", type=int, dest='max_padding',
+                          default=True, metavar="NUM_BYTES",
+                          help=ARGS_HELP["--max-padding"])
+        gid3.add_argument("--no-max-padding", dest='max_padding',
+                          action="store_const", const=None,
+                          help=ARGS_HELP["--no-max-padding"])
+
         _encodings = ["latin1", "utf8", "utf16", "utf16-be"]
         gid3.add_argument("--encoding", dest="text_encoding", default=None,
                           choices=_encodings, metavar='|'.join(_encodings),
@@ -418,6 +434,7 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
             new_tag = True
 
         save_tag = (self.handleEdits(self.audio_file.tag) or
+                    self.handlePadding(self.audio_file.tag) or
                     self.args.force_update or self.args.convert_version)
 
         self.printAudioInfo(self.audio_file.info)
@@ -436,10 +453,18 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
             printWarning("Writing ID3 version %s" %
                          id3.versionToString(version))
 
+            # DEFAULT_MAX_PADDING is not set up as argument default,
+            # because we don't want to rewrite the file if the user
+            # did not trigger that explicitly:
+            max_padding = self.args.max_padding
+            if max_padding is True:
+                max_padding = DEFAULT_MAX_PADDING
+
             self.audio_file.tag.save(
                     version=version, encoding=self.args.text_encoding,
                     backup=self.args.backup,
-                    preserve_file_time=self.args.preserve_file_time)
+                    preserve_file_time=self.args.preserve_file_time,
+                    max_padding=max_padding)
 
         if self.args.rename_pattern:
             # Handle file renaming.
@@ -475,13 +500,6 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
                       info.mp3_header.sample_freq, info.mp3_header.mode))
             printMsg("-" * 79)
 
-    def _getDefaultNameForImage(self, image_frame, suffix=""):
-        name_str = image_frame.picTypeToString(image_frame.picture_type)
-        if suffix:
-            name_str += suffix
-        name_str = "%s.%s" % (name_str, image_frame.mime_type.split("/")[1])
-        return name_str
-
     def _getDefaultNameForObject(self, obj_frame, suffix=""):
         if obj_frame.filename:
             name_str = obj_frame.filename
@@ -507,6 +525,7 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
             printMsg("%s: %s" % (boldText("title"), title))
             printMsg("%s: %s" % (boldText("artist"), artist))
             printMsg("%s: %s" % (boldText("album"), album))
+            printMsg("%s: %s" % (boldText("album artist"), tag.album_artist))
 
             for date, date_label in [
                     (tag.release_date, "release date"),
@@ -617,11 +636,11 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
                         if not os.path.isdir(img_path):
                             raise IOError("Directory does not exist: %s" %
                                           img_path)
-                        img_file = self._getDefaultNameForImage(img)
+                        img_file = img.makeFileName()
+                        name, ext = os.path.splitext(img_file)
                         count = 1
                         while os.path.exists(os.path.join(img_path, img_file)):
-                            img_file = self._getDefaultNameForImage(img,
-                                                                    str(count))
+                            img_file = "".join(["%s%d" % (name, count), ext])
                             count += 1
                         printWarning("Writing %s..." % os.path.join(img_path,
                                                                     img_file))
@@ -678,9 +697,14 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
                 printMsg("-" * 79)
                 printMsg("%d ID3 Frames:" % len(tag.frame_set))
                 for fid in tag.frame_set:
-                    num_frames = len(tag.frame_set[fid])
+                    frames = tag.frame_set[fid]
+                    num_frames = len(frames)
                     count = " x %d" % num_frames if num_frames > 1 else ""
-                    printMsg("%s%s" % (fid, count))
+                    total_bytes = sum(frame.header.data_size + frame.header.size
+                                      for frame in frames)
+                    printMsg("%s%s (%d bytes)" % (fid, count, total_bytes))
+                printMsg("%d bytes unused (padding)" %
+                         (tag.file_info.tag_padding_size, ))
         else:
             raise TypeError("Unknown tag type: " + str(type(tag)))
 
@@ -705,6 +729,14 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
 
         return status
 
+    def handlePadding(self, tag):
+        max_padding = self.args.max_padding
+        if max_padding is None or max_padding is True:
+            return False
+        padding = tag.file_info.tag_padding_size
+        needs_change = padding > max_padding
+        return needs_change
+
     def handleEdits(self, tag):
         retval = False
 
@@ -728,6 +760,7 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
         for (what, arg, setFunc) in (
                 ("artist", self.args.artist, tag._setArtist),
                 ("album", self.args.album, tag._setAlbum),
+                ("album artist", self.args.album_artist, tag._setAlbumArtist),
                 ("title", self.args.title, tag._setTitle),
                 ("genre", self.args.genre, tag._setGenre),
                 ("release date", self.args.release_date, tag._setReleaseDate),
@@ -769,6 +802,15 @@ optional. For example, 2012-03 is valid, 2012--12 is not.
             tag.track_num = track_info
             retval = True
 
+        # --track-offset
+        if self.args.track_offset:
+            offset = self.args.track_offset
+            tag.track_num = (tag.track_num[0] + offset, tag.track_num[1])
+            printWarning("%s track info by %d: %d" %
+                         ("Incrementing" if offset > 0 else "Decrementing",
+                         offset, tag.track_num[0]))
+            retval = True
+
         # --disc-num, --disc-total
         disc_info = _checkNumberedArgTuples(tag.disc_num,
                                             (self.args.disc_num,
@@ -926,6 +968,11 @@ def _getTemplateKeys():
 ARGS_HELP = {
         "--artist": "Set the artist name.",
         "--album": "Set the album name.",
+        "--album-artist": "Set the album artist name. '%s', for "
+                          "example. Another example is collaborations when the "
+                          "track artist might be 'Eminem featuring Proof' "
+                          "the album artist would be 'Eminem'." %
+                          core.VARIOUS_ARTISTS,
         "--title": "Set the track title.",
         "--track": "Set the track number. Use 0 to clear.",
         "--track-total": "Set total number of tracks. Use 0 to clear.",
@@ -1038,6 +1085,14 @@ ARGS_HELP = {
         "--remove-frame": "Remove all frames with the given ID. This option "
                           "may be specified multiple times.",
 
+        "--max-padding": "Shrink file if tag padding (unused space) exceeds "
+                         "the given number of bytes. "
+                         "(Useful e.g. after removal of large cover art.) "
+                         "Default is 64 KiB, file will be rewritten with "
+                         "default padding (1 KiB) or max padding, whichever "
+                         "is smaller.",
+        "--no-max-padding": "Disable --max-padding altogether.",
+
         "--force-update": "Rewrite the tag despite there being no edit "
                           "options.",
         "--verbose": "Show all available tag data",
@@ -1053,5 +1108,7 @@ ARGS_HELP = {
                     "variables: " + _getTemplateKeys(),
         "--preserve-file-times": "When writing, do not update file "
                                  "modification times.",
+        "--track-offset": "Increment/decrement the track number by [-]N. "
+                          "This option is applied after --track=N is set.",
 }
 
